"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.responseInterceptor = void 0;
const zlib = require("zlib");
/**
 * Intercept responses from upstream.
 * Automatically decompress (deflate, gzip, brotli).
 * Give developer the opportunity to modify intercepted Buffer and http.ServerResponse
 *
 * NOTE: must set options.selfHandleResponse=true (prevent automatic call of res.end())
 */
function responseInterceptor(interceptor) {
    return function proxyRes(proxyRes, req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            const originalProxyRes = proxyRes;
            let buffer = Buffer.from('', 'utf-8');
            // decompress proxy response
            const _proxyRes = decompress(proxyRes, proxyRes.headers['content-encoding']);
            // concat data stream
            _proxyRes.on('data', (chunk) => (buffer = Buffer.concat([buffer, chunk])));
            _proxyRes.on('end', () => __awaiter(this, void 0, void 0, function* () {
                // set original content type from upstream
                res.setHeader('content-type', originalProxyRes.headers['content-type'] || '');
                // call interceptor with intercepted response (buffer)
                const interceptedBuffer = Buffer.from(yield interceptor(buffer, originalProxyRes, req, res));
                // set correct content-length (with double byte character support)
                res.setHeader('content-length', Buffer.byteLength(interceptedBuffer, 'utf-8'));
                res.write(interceptedBuffer);
                res.end();
            }));
            _proxyRes.on('error', (error) => {
                res.end(`Error fetching proxied request: ${error.message}`);
            });
        });
    };
}
exports.responseInterceptor = responseInterceptor;
/**
 * Streaming decompression of proxy response
 * source: https://github.com/apache/superset/blob/9773aba522e957ed9423045ca153219638a85d2f/superset-frontend/webpack.proxy-config.js#L116
 */
function decompress(proxyRes, contentEncoding) {
    let _proxyRes = proxyRes;
    let decompress;
    switch (contentEncoding) {
        case 'gzip':
            decompress = zlib.createGunzip();
            break;
        case 'br':
            decompress = zlib.createBrotliDecompress();
            break;
        case 'deflate':
            decompress = zlib.createInflate();
            break;
        default:
            break;
    }
    if (decompress) {
        _proxyRes.pipe(decompress);
        _proxyRes = decompress;
    }
    return _proxyRes;
}
